package io.lumine.cosmetics.constants;

import io.lumine.cosmetics.api.cosmetics.Cosmetic;
import io.lumine.cosmetics.types.back.BackAccessory;
import io.lumine.cosmetics.types.chat.ChatColor;
import io.lumine.cosmetics.types.chat.Sticker;
import io.lumine.cosmetics.types.effects.Effect;
import io.lumine.cosmetics.types.gestures.Gesture;
import io.lumine.cosmetics.types.equipment.hats.Hat;
import io.lumine.cosmetics.types.modelengine.MEGAccessory;
import io.lumine.cosmetics.types.equipment.offhand.Offhand;
import io.lumine.cosmetics.types.pets.Pet;
import io.lumine.cosmetics.types.sprays.Spray;

import java.util.HashMap;
import java.util.Map;

public class CosmeticType {

    private static final Map<Class<? extends Cosmetic>, CosmeticConstant> constants = new HashMap<>();
    private static final Map<String, Class<? extends Cosmetic>> lookupByType = new HashMap<>();

    static {
        register(BackAccessory.class, new CosmeticConstant("BACK", "backs"));
        register(Hat.class, new CosmeticConstant("HAT", "hats"));
        register(MEGAccessory.class, new CosmeticConstant("ACCESSORY", "accessories"));
        register(Pet.class, new CosmeticConstant("PET", "pets"));
        register(Spray.class, new CosmeticConstant("SPRAY", "sprays"));
        register(Offhand.class, new CosmeticConstant("OFFHAND", "offhands"));
        register(Gesture.class, new CosmeticConstant("GESTURE", "gestures"));
        register(ChatColor.class, new CosmeticConstant("CHATCOLOR", "chatcolors"));
        register(Sticker.class, new CosmeticConstant("STICKER", "stickers"));
        register(Effect.class, new CosmeticConstant("EFFECT", "effects"));
    }

    public static void register(Class<? extends Cosmetic> clazz, CosmeticConstant cosmeticConstant) {
        constants.put(clazz, cosmeticConstant);
        lookupByType.put(cosmeticConstant.type(), clazz);
    }

    public static Class<? extends Cosmetic> get(String type) {
        return lookupByType.get(type);
    }

    public static CosmeticConstant get(Class<? extends Cosmetic> tClass) {
        return constants.get(tClass);
    }

    public static String type(Class<? extends Cosmetic> tClass) {
        return get(tClass).type();
    }

    public static String folder(Class<? extends Cosmetic> tClass) {
        return get(tClass).folder();
    }

    public record CosmeticConstant(String type, String folder) {}

}
