package io.lumine.cosmetics.menus;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.constants.CosmeticType;
import io.lumine.cosmetics.logging.MCLogger;
import io.lumine.cosmetics.players.Profile;
import io.lumine.mythic.api.skills.placeholders.PlaceholderString;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.utils.config.properties.types.MenuProp;
import io.lumine.mythic.bukkit.utils.menu.EditableMenuBuilder;
import io.lumine.mythic.bukkit.utils.menu.ReloadableMenu;
import lombok.Getter;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;

public abstract class CosmeticMenu<T> extends ReloadableMenu<T> {

    @Getter protected MCCosmeticsPlugin plugin;
    @Getter protected MenuManager menuManager;
    
    public CosmeticMenu(MCCosmeticsPlugin core, MenuManager manager, MenuProp menu) {
        super(menu);
        this.plugin = core;
        this.menuManager = manager;
    }
    
    public CosmeticMenu(MCCosmeticsPlugin core, MenuManager manager, MenuProp menu, boolean buildOnOpen) {
        super(menu, buildOnOpen);
        this.plugin = core;
        this.menuManager = manager;
    }
    
    public void openMenu(Player player) {
        Profile profile = plugin.getProfiles().getProfile(player);

        //boolean b = profile.getHatIsActive();
        
        //profile.setHatIsActive(false);
        //MainMenu.playMenuClick(player);
        //menu.open(player, state);
        //profile.setHatIsActive(b);
    }

    public void buildAdvancedButtonOptions(EditableMenuBuilder<?> builder) {

        final var display = builder.getBuilder().getTitleFunc().apply(null);
        builder.getBuilder().title((input) -> {
            return PlaceholderString.of(display).get();
        });

        builder.getIcons().values().forEach(icon -> {
            final var config = icon.getConfigurationSection();

            if(config.isSet("Model") && !config.isInt("Model")) {
                var model = config.getString("Model");

                if(MythicBukkit.inst().getCompatibility().getCrucible().isPresent() && MythicBukkit.inst().getCompatibility().getCrucible().get().isImplicitlyGenerating()) {
                    final var generationFeedback = MythicBukkit.inst().getCompatibility().getCrucible().get().addItemToGeneration(plugin.getConfiguration().getMythicPack(), model);

                    if(generationFeedback.getItemModel().isPresent()) {
                        var itemModelKey = generationFeedback.getItemModel().get();
                        icon.getBuilder().itemModel(itemModelKey);
                    }
                }
            }
        });

        builder.getIcon("NEXT_PAGE").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                playMenuClick(player);
                nextPage(player);
            });
        });
        builder.getIcon("PREVIOUS_PAGE").ifPresent(icon -> {
            icon.getBuilder().click((context,player) -> {
                playMenuClick(player);
                previousPage(player);
            });
        });
    }

    public static void playMenuClick(Player player) {
        player.playSound(player.getLocation(), "entity.chicken.egg", 1F, 1F);
    }
}
