package io.lumine.cosmetics.menus;

import com.google.common.collect.Lists;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.cosmetics.Cosmetic;
import io.lumine.cosmetics.types.MCCosmeticsManager;
import io.lumine.cosmetics.players.Profile;
import io.lumine.mythic.bukkit.utils.CollectionUtils;
import io.lumine.mythic.bukkit.utils.logging.Log;
import io.lumine.mythic.bukkit.utils.plugin.ReloadableModule;
import io.lumine.mythic.core.menus.ColorPickerMenu;
import lombok.Getter;

import java.util.Collection;
import java.util.Collections;
import java.util.List;

public class MenuManager extends ReloadableModule<MCCosmeticsPlugin> {

    @Getter private CustomizeMenu customizeMenu;
    @Getter private GenericSelectionMenu selectionMenu;
    @Getter private SelectColorMenu colorPickerMenu;

    public MenuManager(MCCosmeticsPlugin plugin) {
        super(plugin);
    }

    @Override
    public void load(MCCosmeticsPlugin plugin) {

        if(customizeMenu == null) {
            customizeMenu = new CustomizeMenu(plugin,this);
            selectionMenu = new GenericSelectionMenu(plugin,this);
            colorPickerMenu = new SelectColorMenu(plugin,this);
        }
        
        customizeMenu.reload();
        selectionMenu.reload();
        colorPickerMenu.reload();
    }

    @Override
    public void unload() {}

    public void openCosmeticMenu(MCCosmeticsManager manager, Profile profile) {
        final var context = new SelectionMenuData(manager, profile);
        final List<Cosmetic> cosmetics = Lists.newArrayList(manager.getAllCosmetics());

        sortAndFilterCosmetics(manager, profile, context, cosmetics);

        if(manager.getMenu() == null) {
            getSelectionMenu().open(profile.getPlayer(), context, cosmetics);
        } else {
            manager.getMenu().open(profile.getPlayer(), context, cosmetics);
        }
    }

    public void openCosmeticMenu(MCCosmeticsManager manager, Profile profile, SelectionMenuData context) {
        final List<Cosmetic> cosmetics = Lists.newArrayList(manager.getAllCosmetics());

        sortAndFilterCosmetics(manager, profile, context, cosmetics);

        if(manager.getMenu() == null) {
            getSelectionMenu().open(profile.getPlayer(), context, cosmetics);
        } else {
            manager.getMenu().open(profile.getPlayer(), context, cosmetics);
        }
    }

    private void sortAndFilterCosmetics(MCCosmeticsManager manager, Profile profile, SelectionMenuData context, List<Cosmetic> cosmetics) {

        if(context.hasFilter()) {
            final var filter = context.getFilter();

            final Collection<String> filteredNames = CollectionUtils.filterMatches(filter, cosmetics.stream().map(cosmetic -> cosmetic.getKey()).toList(),2);

            cosmetics.removeIf(cosmetic -> {
                boolean remove = true;

                if(filteredNames.contains(cosmetic.getKey())) {
                    remove = false;
                }

                Log.info("filter {0} {1}", cosmetic.getKey(), remove);
                return remove;
            });
        }
        if(context.isShowingUncollected() == false) {
            cosmetics.removeIf(cosmetic -> {
                var bool = !cosmetic.has(profile);
                Log.info("cosmetic {0} {1}", cosmetic.getKey(), bool);
                return bool;
            });
        }

        Collections.sort(cosmetics, (i1, i2) -> {
            return i1.getKey().compareToIgnoreCase(i2.getKey());
        });
    }

    public void openColorPicker(MCCosmeticsManager manager, Profile profile, SelectionMenuData selectionMenuData, Cosmetic cosmetic) {
        final var selectColorContext = new SelectColorContext(manager, profile, selectionMenuData, cosmetic, cosmetic.getColor());
        colorPickerMenu.open(profile.getPlayer(), selectColorContext);
    }

    public void openColorPicker(MCCosmeticsManager manager, Profile profile, SelectColorContext selectColorContext) {
        colorPickerMenu.open(profile.getPlayer(), selectColorContext);
    }
}
