package io.lumine.cosmetics.skills.mechanics;

import io.lumine.cosmetics.MCCosmeticsPlugin;
import io.lumine.cosmetics.api.cosmetics.Cosmetic;
import io.lumine.cosmetics.api.cosmetics.ItemCosmetic;
import io.lumine.cosmetics.constants.CosmeticType;
import io.lumine.cosmetics.logging.MCLogger;
import io.lumine.cosmetics.players.Profile;
import io.lumine.cosmetics.types.chat.Sticker;
import io.lumine.mythic.api.adapters.AbstractEntity;
import io.lumine.mythic.api.skills.ITargetedEntitySkill;
import io.lumine.mythic.api.skills.SkillMetadata;
import io.lumine.mythic.api.skills.SkillResult;
import io.lumine.mythic.bukkit.BukkitAdapter;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.events.MythicMechanicLoadEvent;
import io.lumine.mythic.core.utils.annotations.MythicMechanic;
import org.bukkit.entity.Player;
import org.bukkit.inventory.ItemStack;

import java.util.Optional;

@MythicMechanic(name="cosmeticUnlockEffect",aliases={},description="Plays the cosmetic unlock effect")
public class UnlockEffect implements ITargetedEntitySkill {

    private final MCCosmeticsPlugin plugin;

    protected String cosmeticType;
    protected String cosmeticKey;

    public UnlockEffect(MCCosmeticsPlugin plugin, MythicMechanicLoadEvent loader) {
        this.plugin = plugin;
        final var config = loader.getConfig();

        this.cosmeticType = config.getString(new String[]{"type", "t"}, "HAT");
        this.cosmeticKey = config.getString(new String[]{"key", "k", "cosmetic", "c", "id"}, null);
    }

    @Override
    public SkillResult castAtEntity(SkillMetadata skillMetadata, AbstractEntity abstractEntity) {
        if(!abstractEntity.isPlayer()) {
            return SkillResult.INVALID_TARGET;
        }
        final var player = (Player) abstractEntity.getBukkitEntity();
        final var profile = plugin.getProfiles().getProfile(player);

        final var maybeManager = plugin.getCosmetics().getManager(cosmeticType);
        if(maybeManager.isEmpty()) {
            MCLogger.error("Cosmetic Type not found");
            return SkillResult.INVALID_CONFIG;
        }
        final var manager = maybeManager.get();

        final Optional<Cosmetic> maybeCosmetic = manager.getCosmetic(cosmeticKey);
        if(maybeCosmetic.isEmpty()) {
            MCLogger.error("Cosmetic not found");
            return SkillResult.INVALID_CONFIG;
        }
        final var cosmetic = maybeCosmetic.get();

        ItemStack stack;
        if(cosmetic instanceof ItemCosmetic itemCosmetic) {
            stack = itemCosmetic.getCosmetic();
        } else if(cosmetic instanceof Sticker sticker) {
            stack = sticker.getMenuItem();
        } else {
            return SkillResult.INVALID_CONFIG;
        }

        final var aPlayer = BukkitAdapter.adapt(player);

        MythicBukkit.inst().getVolatileCodeHandler().getEntityHandler().playTotemEffect(aPlayer, stack);

        return SkillResult.SUCCESS;
    }
}
