package io.lumine.cosmetics.types.effects;

import io.lumine.cosmetics.api.cosmetics.ColorableCosmetic;
import io.lumine.cosmetics.api.cosmetics.EquippedCosmetic;
import io.lumine.cosmetics.api.cosmetics.ItemCosmetic;
import io.lumine.cosmetics.config.Scope;
import io.lumine.cosmetics.constants.CosmeticType;
import io.lumine.cosmetics.logging.MCLogger;
import io.lumine.cosmetics.types.AbstractCosmetic;
import io.lumine.mythic.api.packs.Pack;
import io.lumine.mythic.api.skills.Skill;
import io.lumine.mythic.bukkit.MythicBukkit;
import io.lumine.mythic.bukkit.utils.config.properties.Property;
import io.lumine.mythic.bukkit.utils.config.properties.types.IntProp;
import io.lumine.mythic.bukkit.utils.config.properties.types.StringListProp;
import io.lumine.mythic.bukkit.utils.config.properties.types.StringProp;
import io.lumine.mythic.bukkit.utils.items.ItemFactory;
import lombok.Getter;
import org.bukkit.inventory.ItemStack;

import java.io.File;

public class Effect extends AbstractCosmetic implements ColorableCosmetic {

	private static final StringListProp SKILLS = Property.StringList(Scope.NONE, "Skills");
	private static final IntProp INTERVAL = Property.Int(Scope.NONE, "Interval");

	@Getter private Skill skill;
	@Getter private int interval;

	public Effect(EffectManager manager, Pack pack, File file, String key) {
	    super(manager, pack, file, CosmeticType.type(Effect.class), key);

		final var skillConfig = SKILLS.fget(file, this);
		this.interval = INTERVAL.fget(file, this);

		final var maybeSkill = MythicBukkit.inst().getSkillManager().getSkill(file, skillConfig);

		if(maybeSkill.isPresent()) {
			this.skill = maybeSkill.get();
		} else {
			MCLogger.error("Failed to parse skill for Effect {0}", getPropertyNode());
		}
	}

}
